#!/usr/bin/env python3
# Python 2 support
from __future__ import print_function

from netsnmp import *
from netsnmp._api import SNMPError
import sys, time

try:
    time.perf_counter()
except:
    time.perf_counter = time.time

if __name__ == '__main__':
    ips = ['archt01', 'archt02', 'archt03', 'archt04', 'archt05']
    ips6 = ['udp6:[fe80::c67:bb2b:dbb4:8c63]', 'udp6:[fe80::ce0b:fd3a:ac06:26a9]', 'udp6:[fe80::b97d:dda5:1b0e:dd2e]', 'udp6:[fe80::44ee:1be2:784b:84ed]', 'udp6:[fe80::ba63:79c3:cfdd:599d]']


    try:
        sys.argv[1]
    except IndexError:
        [sys.argv.append(op) for op in ('get', 'getnext', 'walk')]

    if 'get' in sys.argv[1:]:
        oids = ['.1.3.6.1.2.1.1.1.0', '.1.3.6.1.2.1.1.3.0', '.1.3.6.1.2.1.1.5.0', '.1.3.6.1.2.1.55.1.5.1.8.2', '.1.3.6.1.2.1.25.1.2.0']
        #oids = [oids.pop(),]
        start = time.perf_counter()
        print('SNMP GET on %s' % oids)
        for host in ips:
            try:
                _start = time.perf_counter()
                # Context (automatically closes session)
                with SNMPSession(host, 'public', debug=1) as ss:
                    responses = ss.get(oids)
                    print('[%s] received %d responses in %02fms' % (host, len(responses), (time.perf_counter()-_start)*1000))
                    # example including conversion of hex-string:
                    for oid in responses:
                        _type, _value = snmp_hex2str(oid[TYPE], oid[VALUE]) if oid[TYPE]=="Hex-STRING" else (oid[TYPE], oid[VALUE])
                        print("[%s] %s = %s: %s" % (host, oid[OID], _type, _value))
    
                # Non-context
                #ss = SNMPSession(host, 'public', debug=1)
                #responses = ss.get(oids)
                #print('[%s] received %d responses in %02fms' % (host, len(responses), (time.perf_counter()-_start)*1000))
                #[print("[%s] %s = %s: %s" % (host, oid[OID], oid[TYPE], oid[VALUE])) for oid in responses]
                #ss.close()
    
            except SNMPError as e:
                # This would ensure session is closed incase context was not used
                try:
                    if ss.is_alive():
                        ss.close()
                except:
                    pass
    
                print("[%s]" % host, repr(e))
                continue
    
        print("Total time taken: %.03fms" % ((time.perf_counter()-start)*1000))
        print()

    if 'getnext' in sys.argv[1:]:
        oids = ['.1.3.6.1.2.1.1', '.1.3.6.1.2.1.1.2.0', '.1.3.6.1.2.1.1.4.0']
        start = time.perf_counter()
        print('SNMP GETNEXT on %s' % oids)
        for host in ips:
            try:
                _start = time.perf_counter()
                with SNMPSession(host, 'public', debug=1) as ss:
                    responses = ss.getnext(oids)
                    print('[%s] received %d responses in %02fms' % (host, len(responses), (time.perf_counter()-_start)*1000))
                    [print("[%s] %s = %s: %s" % (host, oid[OID], oid[TYPE], oid[VALUE])) for oid in responses]
    
            except SNMPError as e:
                try:
                    if ss.is_alive():
                        ss.close()
                except:
                    pass
    
                print("[%s]" % host, repr(e))
                continue
    
        print("Total time taken: %.03fms" % ((time.perf_counter()-start)*1000))
        print()

    if 'walk' in sys.argv[1:]:
        oids = ['.1.3.6.1.2.1.1']
        start = time.perf_counter()
        print('SNMP WALK on %s' % oids)
        for host in ips[:1]:
            try:
                _start = time.perf_counter()
                with SNMPSession(host, 'public', debug=1) as ss:
                    # SNMPSession.walk is a generator
                    [print("[%s] %s = %s: %s" % (host, response[OID], response[TYPE], response[VALUE])) for response in ss.walk(oids)]
    
            except SNMPError as e:
                try:
                    if ss.is_alive():
                        ss.close()
                except:
                    pass
    
                print("[%s]" % host, repr(e))
                continue
    
        print("Total time taken: %.03fms" % ((time.perf_counter()-start)*1000))
        print()



import binascii, struct

def snmp_hex2str(type, value):
    """
    Helper func to convert various types of hex-strings, determined by length
    """

    # Remove any surrounding quotes
    if value[0]=='"' and value[-1]=='"':
        # '"AB'" -> 'AB'
        _hexstr = value[1:-1]
    else:
        _hexstr = value

    _hexstrl = len(_hexstr)

    if _hexstrl==18:
        # Return cleanly formatted MAC address, no conversion nescessary
        type = "MacAddress"
        value = '%s:%s:%s:%s:%s:%s' % tuple(_hexstr.split())

    elif _hexstrl==12 or _hexstrl==4:
        ## Convert octal IpAddress
        # example input: 'C0 A8 01 01 ' or 'DV8W'
        # C0 = 192
        # A8 = 168
        # 01 = 1
        # 01 = 1
        type = "IpAddress"
        if _hexstrl==4:
            # Convert ascii-alike strings
            value = '%d.%d.%d.%d' % tuple((ord(char) for char in _hexstr))
        else:
            # Convert hex strings
            value = '%d.%d.%d.%d' % tuple((ord(binascii.unhexlify(part)) for part in _hexstr.split()))

    elif _hexstrl==33:
        ## Convert DateAndTime
        # example input: '07 DF 0C 0E 16 15 09 00 2D 05 00 '
        # 07 DF = year
        # 0C = month
        # 0E = day
        # 16 = hour
        # 15 = minutes
        # 09 = seconds
        # 00 = deci-seconds
        # 2D = direction from UTC ('+'/'-'), e.g. chr(45)==str('-')
        # 05 = hours from UTC
        # 00 = minutes from UTC
        type = "DateAndTime"

        # given above example, unhexlify "07DF" (b'\x07\xdf') then unpack as big-endian unsigned short (2015)
        year = struct.unpack('>H', binascii.unhexlify("".join(_hexstr.split()[:2])))[0]

        (month, day, hour, minute, second, decisecond,
        utcdir, utchour, utcminute) = (ord(binascii.unhexlify(part)) for part in _hexstr.split()[2:])

        # zero padded hour, minute, second
        value = '%d-%d-%d,%0#2d:%0#2d:%0#2d.%d,%s%s:%s' % (
                year, month, day, hour, minute, second, decisecond, chr(utcdir), utchour, utcminute)

    return (type, value)



#!/usr/bin/env python3
"""
https://github.com/xstaticxgpx/netsnmp-py3/

Examples:

SNMP GET
---
    >>> with netsnmp.SNMPSession('archt01', 'public') as ss:
    ...     ss.get(['.1.3.6.1.2.1.1.1.0', '.1.3.6.1.2.1.1.3.0', '.1.3.6.1.2.1.1.5.0'])
    ... 
    [('.1.3.6.1.2.1.1.1.0', 'STRING', '"Linux archt01 4.3.0-1-ck #1 SMP PREEMPT Sun Nov 15 13:24:29 EST 2015 x86_64"'), ('.1.3.6.1.2.1.1.3.0', 'Timeticks', '1:11:36:30.56'), ('.1.3.6.1.2.1.1.5.0', 'STRING', '"archt01"')]

SNMP GETNEXT
---
    >>> with netsnmp.SNMPSession('archt01', 'public') as ss:
    ...     ss.getnext(['.1.3.6.1.2.1.1.1', '.1.3.6.1.2.1.1.2.0', '.1.3.6.1.2.1.1.4.0'])
    ... 
    [('.1.3.6.1.2.1.1.1.0', 'STRING', '"Linux archt01 4.3.0-1-ck #1 SMP PREEMPT Sun Nov 15 13:24:29 EST 2015 x86_64"'), ('.1.3.6.1.2.1.1.3.0', 'Timeticks', '1:11:39:35.05'), ('.1.3.6.1.2.1.1.5.0', 'STRING', '"archt01"')]

SNMP WALK (load averages)
---
    >>> with netsnmp.SNMPSession('archt01', 'public') as ss:
    >>>     [response for response in ss.walk(['.1.3.6.1.4.1.2021.10.1.3'])]
    ... 
    [('.1.3.6.1.4.1.2021.10.1.3.1', 'STRING', '"0.37"'), ('.1.3.6.1.4.1.2021.10.1.3.2', 'STRING', '"0.25"'), ('.1.3.6.1.4.1.2021.10.1.3.3', 'STRING', '"0.29"')]

"""

#import netsnmp._api as netsnmp
from . import _api as netsnmp
from ._hex import snmp_hex2str

class SNMPRuntimeError(netsnmp.SNMPError): # pylint: disable=no-init
    """
    Raised by SNMPSession methods on return code error
    """
    pass

SNMP_VER = {
    '1': 0,
    '2c': 1,
    '3': 3,
}

SNMP_ERR = [
    'NOSUCHINSTANCE',
    'ENDOFMIBVIEW',
    'NOSUCHOBJECT',
    'ERROR',
]

# Response tuple pointers
OID = 0
TYPE = 1
VALUE = 2

def oid_outside_tree(oid1, oid2):
    """
    Return True if oid2 outside of oid1 tree
    Used by SNMPSession.walk
    """
    try:
        if oid2.startswith(oid1):
            return False
        return True
    except:
        return True


class SNMPSession(object):
    """
    Session based, thread-safe interface
    """
    def __init__(self, peername, community, version=SNMP_VER['2c'], timeout=0.5, retries=1, debug=0): # pylint: disable=line-too-long
        self.debug = debug # 1 for partial debugging, 2 for full NETSNMP debugging
        self.version = version
        self.timeout = int(timeout*1000000) # seconds converted to microseconds
        self.retries = retries
        self.peername = peername
        self.community = community
        # Define session
        self.sess_ptr = netsnmp.create_session(self.version, self.timeout, self.retries,
                                               self.community, self.peername, self.debug)
        # Internal variables
        self._alive = True
        self._next = False

    def __enter__(self):
        """
        Support context (with SNMPSession(..) as ss..)
        """
        return self

    def __exit__(self, *args):
        self.close()

    def close(self):
        """
        Close session (clear file descriptors)
        """
        if netsnmp.close_session(self):
            self._alive = False

    def is_alive(self):
        return self._alive

    def get(self, oids):
        """
        Wrap netsnmp._api.get C function
        """
        # Define list to be populated by C API get()
        responses = []
        self._next = False

        # netsnmp.get(SNMPSession(), oids=[oid,..], responses=[])
        # Response information is appended as a tuple of (OID, TYPE, VALUE) to responses
        # Return 1 on success, possibly raises SNMPError() exception
        _rc = netsnmp.get(self, oids, responses)
        # Sanity check rc
        if not _rc:
            raise SNMPRuntimeError("Invalid return code", _rc)
        return responses

    def getnext(self, oids):
        """
        Wrap netsnmp._api.get C function for GETNEXT
        """
        responses = []
        # Flag for GETNEXT
        self._next = True

        _rc = netsnmp.get(self, oids, responses)
        if not _rc:
            raise SNMPRuntimeError("Invalid return code", _rc)
        return responses

    def walk(self, oids):
        """
        Generator implementation of snmpwalk using self.getnext
        """
        if type(oids) is str:
            oids = [oids]
        for oid in oids:
            next_oid = oid
            while True:
                response = self.getnext(next_oid)[0]
                if oid_outside_tree(oid, response[OID]):
                    break
                elif response[TYPE] == "ENDOFMIBVIEW":
                    break
                elif response[TYPE] in SNMP_ERR:
                    raise netsnmp.SNMPError(response[TYPE], response[VALUE])
                next_oid = response[OID]
                yield response
        raise StopIteration



#!/usr/bin/python3

from netsnmp._api import get_async
from async_devtypes import SNMP_DEVTYPES
#import cx_Oracle
import zmq
import redis, hiredis
import logging, logging.handlers
import random, sys, threading, time
import multiprocessing as mp

# Python 2 support
try:
    import queue
except:
    import Queue as queue

try:
    time.perf_counter()
except:
    time.perf_counter = time.time

# command name
cmdname = sys.argv[0]

# configure short loglevel names
logging._levelToName = {
    logging.CRITICAL: 'CRI',
    logging.ERROR:    'ERR',
    logging.WARNING:  'WAR',
    logging.INFO:     'INF',
    logging.DEBUG:    'DBG',
    logging.NOTSET:   'NOSET',
}

# DB definitions
DB = {
    "db": {
        'server': 'host',
        'port'  : '1521',
        'name'  : 'name',
        'user'  : 'user',
        'pass'  : 'pass',
    },
}

### Global definitions
#BUG: udp sendto failures when 49116 > MAX_WORKERS*MAX_PER_WORKER > 49112
# limitation lies somewhere between 49112 and 49116(??) - getting EPERM (1)... kernel dropping?
#http://comments.gmane.org/gmane.comp.security.firewalls.netfilter.devel/29993
# Ceiling is around 8 * 4096 = 32768 max requests at any given time
# or 4096*12 = 3072*16 = 49152
MAX_WORKERS = 4
MAX_PER_WORKER = 4096

# Number of ZMQ PULL processes to spawn
ZMQ_PROCESSORS = 2

# Time to pause for ZMQ initialization (Seconds)
ZMQ_PAUSE=0.01

# ZMQ High water mark
ZMQ_HWM=10000000

# ZMQ endpoints
ZMQ_IN  = "ipc:///tmp/%s_in" % cmdname
ZMQ_OUT = "ipc:///tmp/%s_out" % cmdname

## SNMP
# SNMP timeout is in milliseconds
SNMP_TIMEOUT=1250
SNMP_RETRIES=1
SNMP_TIMEOUT_DELTA=MAX_WORKERS

# Response callback message intake/processing
def ZMQProcessor(success, timeout, oidcount):
    """
    Intake work via ZeroMQ IPC socket and queue for processing after _sentinel is signaled
    """
    # ZeroMQ Message Frame pointers ([OP, HOST, DEVTYPE, OIDS..])
    OP=0
    HOST=1
    DEVTYPE=2
    OIDS=3
    # Local counters, rolled up into mp.Value at end
    _type_count = {}
    _success = 0
    _timeout = 0
    _oidcount = 0

    # Processor ZMQ PULL socket
    incoming = zmq.Context().socket(zmq.PULL)
    incoming.setsockopt(zmq.RCVHWM, ZMQ_HWM)
    incoming.connect(ZMQ_OUT)

    log.debug('Starting up...')

    ## Intake

    _redis = redis.Redis(host='127.0.0.1')
    with _redis.pipeline() as _redispipe:
        i=0
        # Pull messages via ZMQ, process and ship to Redis
        while True:
            response = [frame.decode() for frame in incoming.recv_multipart()]
            if response[OP] == '_sentinel':
                log.debug('Shutting down ZMQ_PULL socket...')
                incoming.close()
                break

            elif response[OP] == '1':
                _success+=1
 
                try:
                    _type_count[response[DEVTYPE]]+=1
                except KeyError:
                    # Need to define
                    _type_count[response[DEVTYPE]]=1

                # Parse OIDs
                _vars = SNMP_DEVTYPES[response[DEVTYPE]].parse_oids(response[OIDS:])
                _oidcount+=len(_vars)
                #log.debug(_vars)
                try: 
                    #log.debug("%s [%s] %s", response[HOST], response[DEVTYPE], vars)
                    # splice if ipv6
                    _redispipe.hmset(response[HOST] if not response[HOST][:4]=="udp6" else response[HOST][6:][:-1],
                                 _vars)
                    i+=1
                except redis.exceptions.RedisError as e:
                    log.debug('redis exception: %s %s:%s', str(e).strip(), response[HOST], _vars)
                    continue
                # Flush redis pipeline periodically
                if i > 4096:
                    _redispipe.execute()
                    i=0

            elif response[OP] == '2':
                _timeout+=1

        _redispipe.execute()

    # Counter rollup
    with success.get_lock():
        success.value+=_success
    with timeout.get_lock():
        timeout.value+=_timeout
    with oidcount.get_lock():
        oidcount.value+=_oidcount

    #log.info('Finished processing %d responses in %.3fs' % (qsize, elapsed))
    log.info('Processed %d responses', (_success+_timeout))
    log.debug('%s', _type_count)

# Messaging Pipeline
def ZMQStreamer(running):
    """
    Proxy ZeroMQ via pipeline (zmq.STREAMER)
    """

    incoming = zmq.Context().socket(zmq.PULL)
    incoming.setsockopt(zmq.RCVHWM, ZMQ_HWM*MAX_WORKERS)
    incoming.bind(ZMQ_IN)

    outgoing = zmq.Context().socket(zmq.PUSH)
    outgoing.setsockopt(zmq.SNDHWM, ZMQ_HWM*ZMQ_PROCESSORS)
    outgoing.bind(ZMQ_OUT)

    log.debug('Starting up...')
    with running.get_lock():
        running.value = 1
    zmq.device(zmq.STREAMER, incoming, outgoing)

if __name__ == '__main__':

    #db = DB['db']
    #dbh = cx_Oracle.connect('%s/%s@%s/%s' % (db['user'], db['pass'], db['server'], db['name']))

    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s.%(msecs)03dZ [%(processName)s/%(levelname)s] %(message)s',
                        datefmt='%Y-%m-%dT%H:%M:%S')
    log = logging.getLogger(__name__)

    try:
        _log_queue = queue.Queue()
        log_async  = logging.handlers.QueueHandler(_log_queue)
        log_queue  = logging.handlers.QueueListener(_log_queue, *log.handlers)

        log_queue.start()

        # Overwrite handlers to only utilize QueueHandler()
        log.handlers = [log_async,]
    except AttributeError:
        pass

    #select = dbh.cursor()
    #select.arraysize = 4096

    query = "select something from somewhere"
    community = "public"

    CM_IP = 0
    MODEL = 1

    # Absolute start timer
    _start = time.time()
    # Step timer
    start = time.perf_counter()
    #list(tuple(peername str, community str, devtype str, devtype class instance)..)
    hosts = [(
         "udp6:["+host[CM_IP]+"]" if host[CM_IP][4]==':' else host[CM_IP],
         community,
         host[MODEL] if host[MODEL] in SNMP_DEVTYPES else '__other__',
         SNMP_DEVTYPES[host[MODEL]] if host[MODEL] in SNMP_DEVTYPES else SNMP_DEVTYPES['__other__']
         ) for host in (('archt01', 'other'), ('archt02', 'other'), ('archt03', 'other'), ('archt04', 'other'), ('archt05', 'other'))*200]
    #select.close()
    #dbh.close()
    total = len(hosts)
    end = time.perf_counter()
    log.info('got %d hosts from DB in %.3fms' % (total, (end-start)*1000))

    try:
        ## Global multiprocessing-safe counters
        success = mp.Value('i', 0)
        timeout = mp.Value('i', 0)
        oidcount = mp.Value('i', 0)
        ## ZMQStreamer switch (set to 1 after successful initialization)
        zmq_streamer_running = mp.Value('i', 0)


        # Start ZeroMQ Streamer
        zmq_streamer = mp.Process(target=ZMQStreamer,
                                   args=(zmq_streamer_running,),
                                   name='ZMQStreamer')#,
        zmq_streamer.daemon=True

        zmq_streamer.start()

        # Let the ZMQ sockets start up
        time.sleep(ZMQ_PAUSE)
        if not zmq_streamer_running.value == 1:
            raise RuntimeError("ZMQStreamer failed to initialize")

        # Spin up ZeroMQ Processors
        zmq_processors = []
        for i in range(ZMQ_PROCESSORS):
            zmq_processors.append(
                mp.Process(target=ZMQProcessor,
                            args=(success, timeout, oidcount),
                            name='ZMQProc-%03d' % (i+1))
            )
            zmq_processors[-1].daemon=True
            zmq_processors[-1].start()

        time.sleep(ZMQ_PAUSE)

        # List of multiprocessing.Process() objects (worker processes)
        active_workers = []
        
        # Worker/process id iterator
        p=0
        # Host index range iterator
        i=0
        # While any hosts or workers exist
        start = time.perf_counter()
        _timeout = SNMP_TIMEOUT
        while hosts or active_workers:
            remaining = total-i
            if hosts and len(active_workers) < MAX_WORKERS:
                #_timeout = SNMP_TIMEOUT+random.randint(p%2, SNMP_TIMEOUT_DELTA)
                _upper = i+MAX_PER_WORKER if remaining > MAX_PER_WORKER else i+remaining
                log.debug('Defining process for range %d:%d (%d*%dms)', i, _upper, (SNMP_RETRIES+1), _timeout)

                # Define process(es) which call get_async C function
                # get_async([(str hostname, str community, [str oid,..])..], int timeout_ms, int retries, int ZMQ_HWM, str ZMQ_IN)
                active_workers.append(
                    mp.Process(target=get_async,
                                args=(hosts[:MAX_PER_WORKER], 
                                      _timeout,
                                      SNMP_RETRIES, 
                                      ZMQ_HWM, 
                                      ZMQ_IN))
                )
                active_workers[-1].daemon=True
                active_workers[-1].start()

                del hosts[:MAX_PER_WORKER]
                i+=MAX_PER_WORKER
                if len(active_workers) < MAX_WORKERS:
                    p+=1
                    # test, slow initial startup
                    time.sleep((SNMP_TIMEOUT/MAX_WORKERS)/1000.0)
                    continue
                else:
                    p=0

            pids = [proc.pid for proc in active_workers]
            log.debug('Process PIDs: %s' % pids)

            # Continually monitor progress and re-loop when all workers are finished, or maintain process pool with force_reloop
            while active_workers:
                force_reloop = False
                for proc in active_workers:
                    if proc.is_alive():
                        continue
                    else:
                        active_workers.remove(proc)
                        if proc.exitcode == 0:
                            log.debug('%d - Process finished' % proc.pid)
                        else:
                            log.error('%d - Process failed (%d)' % (proc.pid, proc.exitcode))
                        # if processes take variable amount of time, might want to break and loop to start a new process
                        # this ensures MAX_WORKERS processes active at all times
                        force_reloop = True
                        break
                if force_reloop:
                    break

        # Finally, raise successful completion
        raise SystemExit

    except RuntimeError as e:
        log.critical("%s", e)

    except SystemExit:
        # Exit with success

        end = time.perf_counter()
        elapsed = end-start
        log.info('Polling completed in %.3fs' % elapsed)

        __start = time.perf_counter()
        log.debug('Waiting for ZMQProcessors...')
        # Signal end to ZMQProcessor(s)
        zmq_sentinel = zmq.Context().socket(zmq.PUSH)
        zmq_sentinel.connect(ZMQ_IN)
        for _ in range(ZMQ_PROCESSORS):
            zmq_sentinel.send(b'_sentinel')
        zmq_sentinel.close()

        # Wait for ZMQProcessor(s) to complete
        while zmq_processors:
            for zmq_processor in zmq_processors:
                zmq_processor.join()
                zmq_processors.remove(zmq_processor)

        __elapsed = time.perf_counter()-__start
        log.debug('ZMQProcessors took an additional %.3fs', __elapsed)

        log.info('from %d total hosts got %d timeouts (%.1f%%)' % (total, timeout.value, ((timeout.value/total)*100)))
        log.info('got %d oid responses from %d hosts' % (oidcount.value, success.value))
        log.info('lost %d hosts' % (total-(success.value+timeout.value)))
        log.info('%.2f oids/sec' % (oidcount.value/elapsed))
        log.info('%.2f reqs/sec' % ((success.value+timeout.value)/elapsed))

    finally:

        _end = time.time()
        _elapsed = _end-_start
        log.info('total time taken %.3fs' % _elapsed)

        # Ensure logging is flushed
        try:
            log_queue.stop()
        except:
            pass




from setuptools import setup, Extension, find_packages
import os
import re
import sys

VERSION="0.4"

intree=0

_incdirs = ['./netsnmp']
_libdirs = []

args = sys.argv[:]
for arg in args:
    if '--basedir' in arg:
        basedir = arg.split('=')[1]
        sys.argv.remove(arg)
        intree=1
    if '--incdir' in arg:
        incdir = arg.split('=')[1]
        sys.argv.remove(arg)
        _incdirs.append(incdir)
    if '--libdir' in arg:
        libdir = arg.split('=')[1]
        sys.argv.remove(arg)
        _libdirs.append(libdir)


if intree:
    netsnmp_libs = os.popen(basedir+'/net-snmp-config --libs').read()
    libdir = os.popen(basedir+'/net-snmp-config --build-lib-dirs '+basedir).read()
    incdir = os.popen(basedir+'/net-snmp-config --build-includes '+basedir).read()
    libs = re.findall(r"-l(\S+)", netsnmp_libs)
    libdirs = re.findall(r"-L(\S+)", libdir)
    incdirs = re.findall(r"-I(\S+)", incdir)
else:
    netsnmp_libs = os.popen('net-snmp-config --libs').read()
    libdirs = re.findall(r"-L(\S+)", netsnmp_libs)
    incdirs = []
    libs = re.findall(r" -l(\S+)", netsnmp_libs)

# For _api.h references/travis-ci build
incdirs+=_incdirs
libdirs+=_libdirs

# Asynchronous IPC
libs.append('zmq')
libs.append('czmq')

setup(
    name="netsnmp-py",
    version=VERSION,
    description='Python NET-SNMP Bindings',
    author='Gabe Pacuilla',
    author_email='root@un1x.su',
    url='https://github.com/xstaticxgpx/netsnmp-py3',
    license='MIT License',
    download_url='https://github.com/xstaticxgpx/netsnmp-py3/archive/v%s.tar.gz' % VERSION,
    packages=find_packages(),
    ext_modules = [
       Extension("netsnmp._api",
                 ["netsnmp/session.c",
                  "netsnmp/get.c",
                  "netsnmp/get_async.c",
                  "netsnmp/interface.c",
                  "netsnmp/_api.c"],
                 library_dirs=libdirs,
                 include_dirs=incdirs,
                 libraries=libs)
    ],
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Programming Language :: C',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries'
    ),
)



#!/usr/bin/env python3

from . import SNMPDevice
from netsnmp import snmp_hex2str

class SNMPCiscoDevice(SNMPDevice):

    def __init__(self, oids=[], hexconvert=[]):

        # Ordered list of tuple(name, oid)
        self._oids = []
        
        # Set of OIDs that require manual hex conversion
        self._hexconvert = []

        if oids:
            # Append any extra OIDs passed
            self._oids+=oids
        if hexconvert:
            # Append any extra OIDs requiring conversion
            # get unique set using bitwise or
            self._hexconvert = set(self._hexconvert) | set(hexconvert)

        # Instantiate from parent
        SNMPDevice.__init__(self, oids=self._oids)

    # For demonstration purposes, this is how we can parse oids different in subclass
    def parse_oids(self, response):
        # Responses come back as pipe delimited OID|TYPE|VALUE
        _vars = (var.split('|', maxsplit=2) for var in response)

        # This is where we vary from SNMPDevice
        # e.g., create genexpr to work on _hexconvert oids
        _vars = ((oid,
                  snmp_hex2str(type, value)[1] if oid in self._hexconvert else
                  value.replace('"', '')) for oid, type, value in _vars)

        return {self._oid2str[oid]: value for oid, value in _vars}



#!/usr/bin/env python3

from netsnmp._api import build_pdu

class SNMPDevice(object):

    def __init__(self, oids=[]):

        # Ensure oids is an iterable object
        # We expect oids to be any iterable of (name, oid) tuples
        assert type(oids)==set or \
               type(oids)==list or \
               type(oids)==tuple

        # Boilerplate
        self._str2oid = {}
        self._oid2str = {}
        ## Do dict.update in order to allow subclass additions
        self._str2oid.update(oids)
        self._oid2str.update({oid: name for (name, oid) in oids})

        # Perform set comprehension to remove any duplicate entries
        self._oids = {oid for (name, oid) in oids}

        # Create PDU with the unique OIDs set
        # returns void pointer which is passed to snmp_clone_pdu in get_async.c
        self.pdu = build_pdu(self._oids)

    def parse_oids(self, response):
        # Using this method we can alter parsing logic in subclasses

        # Responses come back as pipe delimited OID|TYPE|VALUE
        # We'll remove double quotes from responses by default
        ## WARNING! ascii-alike hex strings could have " characters within the value
        ## this is handled properly in subclasses.
        _vars = (var.replace('"', '').split('|', 2) for var in response)

        # We don't normally need type
        return {self._oid2str[oid]: value for oid, type, value in _vars}
